[sacred-content="status-icon"][sacred-status="COMPLETED"], .run-completed {
    color: lightgreen;
}

[sacred-content="status-icon"][sacred-status="FAILED"], .run-failed {
    color: indianred;
}

[sacred-content="status-icon"][sacred-status="QUEUED"], .run-queued {
    color: grey;
}

[sacred-content="status-icon"][sacred-status="RUNNING"][sacred-is-alive="true"], .run-alive {
    color: blue;
}

[sacred-content="status-icon"][sacred-status="RUNNING"][sacred-is-alive="false"], .run-dead {
    color: black;
}

[sacred-content="status-icon"][sacred-status="RUNNING"][sacred-is-alive="false"]:after, .run-dead:after {
    color: black;
    content: " ?";
}

[sacred-content="status-icon"][sacred-status="INTERRUPTED"], .run-interrupted {
    color: orange;
}

.detail-page-box {
    overflow: auto;
    height: 35eM;
    width: 98%;
}

.detail-view {
    padding: 1.5eM;
    background-color: rgba(189, 231, 255, 0.50);
    min-height: 30eM;
    max-height: 80%;
}

/* Master-detail view */
#runs tr[role="row"], .cursor-pointer {
    cursor: pointer;
}

td.details-control:before {
    /*background: url('../resources/details_open.png') no-repeat center center;*/
    content: "+";
    cursor: pointer;
}

tr.shown td.details-control:before {
    /*background: url('../resources/details_close.png') no-repeat center center;*/
    content: "-";
}

/* End-master-detail-view*/


/* Query filter */
.query-filter {
    display: inline-block;
    background-color: lightgray;
    padding: 0.5eM;
}

/* Block containing the help message in Query filter */
.block-over {
    position: absolute;
    background-color: white;
    border-radius: 10px;
    border: 1.5px solid black;
    z-index: 1;
    padding: 0.5em;
}